from .imgproc import *
import gentimedist as gen
import cv2, time, sys, cPickle, zmq, os, copy
from math import log
from IPython.parallel import Client
from scipy import stats
import numpy as np

# def findfaceswithtrigger(profile='picluster',threshold=15,n=100,serial='no',show=False):
# 	import sys, cPickle, os
# 	pidfile=open('mypythonpid','w')
# 	cPickle.dump(os.getpid(),pidfile)
# 	pidfile.close()
# 	runstate=open('runstate','w')
# 	cPickle.dump(0,runstate)
# 	runstate.close()
# 	print os.getpid()
# 	#context=zmq.Context()
# 	#pubsocket=context.socket(zmq.PUB)
# 	#port=5556
# 	#pubsocket.bind("tcp://*:%s"%port)
# 	#subsocket=context.socket(zmq.SUB)
# 	if sys.platform=='darwin':
# 		ips=open("/Users/isaac/Code/parastuff/ipaddresses").read()
# 	if sys.platform=='linux2':
# 		ips=open("/home/pi/ipaddresses").read()
# 	splitips=ips.split("\n")
# 	print splitips
# 	#for ip in splitips:
# 	#	if ip:
# 	#		subsocket.connect("tcp://%s:%s"%(ip,port))
# 	#subsocket.setsockopt(zmq.SUBSCRIBE,"need quiet?")
# 
# 	c=Client(profile=profile)
# 	dview=c[:]
# 	numnodes=len(c.ids)
# 	dview.block=False
# 	with dview.sync_imports():
# 		import cv2,sys,time,math,os,cPickle,signal
# 	#dview.execute("mypid=os.getpid()")
# 	def getmypid():
# 		import cPickle
# 		try:
# 			file=open('mypythonpid','r')
# 			mypid=cPickle.load(file)
# 			file.close()
# 		except:
# 			mypid=False
# 		return mypid
# 	def getrunstate():
# 		import cPickle
# 		try:
# 			file=open('runstate','r')
# 			state=cPickle.load(file)
# 			file.close()
# 		except:
# 			state=0
# 		return state
# 	dview["getrunstate"]=getrunstate
# 	dview["getmypid"]=getmypid
# 	dview.execute("mypid=getmypid()")
# 	for i in range(numnodes):
# 			print c[c.ids[i]]['mypid']
# 	print sys.platform
# 	if sys.platform=='darwin':
# 		facedetector=cv2.CascadeClassifier("/usr/local/Cellar/opencv/2.4.9/share/OpenCV/haarcascades/haarcascade_frontalface_alt2.xml")
# 	elif sys.platform=='linux2':
# 		facedetector=cv2.CascadeClassifier("/home/pi/opencv-2.4.9/data/haarcascades/haarcascade_frontalface_alt2.xml")
# 	else: 
# 		print "something went wrong detecting the system"
# 	times=gen.loadtimes()
# 	scale=1.1
# 	dview["scale"]=scale
# 	for i in c.ids:
# 		c[i].execute('if sys.platform=="darwin": haarface=cv2.CascadeClassifier("/usr/local/Cellar/opencv/2.4.9/share/OpenCV/haarcascades/haarcascade_frontalface_alt2.xml")')
# 		c[i].execute('if sys.platform=="linux2": haarface=cv2.CascadeClassifier("/home/pi/opencv-2.4.9/data/haarcascades/haarcascade_frontalface_alt2.xml")')
# 	
# 	numsizes=len(times)
# 	trans=[None]*numsizes
# 	for i in range(numsizes):
# 		trans[i]=int(20*(scale**i))
# 	
# 	mintime=min(times)
# 	times=[thistime-mintime for thistime in times]
# 
# 	print "predicting even distribution..."
# 	def predictdist(numnodes,times,numsizes,c):
# 		timepernode=sum(times)/numnodes
# 		thresholds=[None]*(numnodes+1)
# 		thresholds[0]=0
# 		for i in range(1,numnodes+1):
# 			j=thresholds[i-1]+1
# 			while sum(times[thresholds[i-1]:j])<timepernode and j<numsizes:
# 				j=j+1
# 				#print j
# 				#time.sleep(3)
# 			if sum(times[thresholds[i-1]:j])-timepernode>timepernode-sum(times[thresholds[i-1]:j-1]):
# 				j=j-1 #makes more even, but last few take longest...otherwise even but last is really short
# 			thresholds[i]=j
# 		thresholds[numnodes]=numsizes-1
# 		for i in range(numnodes):
# 			c[c.ids[i]]['thismin']=tuple([trans[thresholds[i]]-1]*2)
# 			c[c.ids[i]]['thismax']=tuple([trans[thresholds[i+1]]+1]*2)
# 	predictdist(numnodes,times,numsizes,c)
# 	
# 	def findfaces(img,scale,serial):
# 		starttime=time.time()
# 		numnodes=len(c.ids)
# 		img=cv2.cvtColor(img,cv2.COLOR_BGR2GRAY)
# 		dview['img']=img
# 		dview.execute('img=img.copy()')
# 		dview.execute("mypid=getmypid()")
# 		runstate=open('runstate','w')
# 		cPickle.dump(1,runstate)
# 		runstate.close()
# 		dview.execute("state=getrunstate()")
# 		#for i in range(numnodes):
# 		#	print c[c.ids[i]]["state"]
# 		dview.execute("if mypid and not state: os.kill(mypid,signal.SIGTSTP)")
# 		dview.execute("exectime=time.time();myfaces=haarface.detectMultiScale(img,scale,4,1,thismin,thismax);exectime=time.time()-exectime;endtime=time.time()",block=False)
# 		faces=set()
# 		thistimes=[None]*numnodes
# 		thistimesremote=[None]*numnodes
# 		for i in range(numnodes):
# 			thistimes[i]=time.time()
# 			print "checking engine "+str(c.ids[i])
# 			thisfaces=c[c.ids[i]]["myfaces"]
# 			thistimesremote[i]=c[c.ids[i]]["exectime"]
# 			print len(thisfaces)
# 			if len(thisfaces)>0:
# 				for face in thisfaces:
# 					faces.add(tuple(face))
# 			thistimes[i]=time.time()-thistimes[i]
# 		print thistimes
# 		print thistimesremote
# 
# 		duptime=time.time()
# 		facelist=list(faces)
# 		donttest=[]
# 		finallist=[]
# 		testlen=len(facelist)
# 		origlist=facelist
# 		testlennew=0
# 		print len(origlist)
# 		while not testlennew==testlen:
# 			testlen=len(facelist)
# 			for first in range(len(facelist)):
# 				if first not in donttest:
# 					for second in range(first+1,len(facelist)):
# 						sub=facelist[first]
# 						j=facelist[second]
# 						var=abs(j[3]-sub[3])/2
# 						if ((j[0]+j[3]/2-var<sub[0]+sub[3]/2) and (j[0]+j[3]/2+var>sub[0]+sub[3]/2)) and ((j[1]+j[2]/2-var<sub[1]+sub[2]/2) and (j[1]+j[2]/2+var>sub[1]+sub[2]/2)) and min([float(j[3])/sub[3],float(sub[3])/j[3]])>1.0/2.0:
# 							donttest.append(first)
# 							donttest.append(second)
# 							#facelist.append(tuple([(j[0]+sub[0])/2,(j[1]+sub[1])/2,(j[2]+sub[2])/2,(j[3]+sub[3])/2]))
# 							inter=tuple([(j[0]+sub[0])/2,(j[1]+sub[1])/2,(j[2]+sub[2])/2,(j[3]+sub[3])/2])
# 							finallist.append(inter)
# 							break
# 					if first not in donttest:
# 						finallist.append(facelist[first])
# 			testlennew=len(finallist)
# 			facelist=finallist
# 			finallist=[]
# 			donttest=[]
# 		finallist=facelist
# 		duptime=time.time()-duptime
# 		print len(finallist)
# 		starttime=time.time()-starttime
# 		print "min/avg/max/regtime/tottime: "+str(min(thistimesremote))+"/"+str(sum(thistimesremote)/numnodes)+"/"+str(max(thistimesremote))+"/"+str(starttime)+"/"+str(sum(thistimesremote))
# 
# 		if serial=='yes':
# 			serialtime=time.time()
# 			facedetector.detectMultiScale(img,scale,4,1)
# 			serialtime=time.time()-serialtime
# 			print serialtime
# 		
# 		dview.execute("if mypid: os.kill(mypid,signal.SIGCONT)")
# 		#for i in range(numnodes):
# 		#	print c[c.ids[i]]['mypid']
# 		runstate=open('runstate','w')
# 		cPickle.dump(0,runstate)
# 		runstate.close()
# 		return finallist
# 	
# 	def calibcam(n,cam):
# 		sumlist=[None]*n
# 		for i in range(n):
# 			retval,img=cam.read()
# 			xstart=img.shape[0]/4
# 			ystart=img.shape[1]/4
# 			sumlist[i]=img[xstart:(xstart*3),ystart:(ystart*3),:].sum()
# 		return sumlist
# 	
# 	cam=cv2.VideoCapture(0)
# 	
# 	retval,img=cam.read()
# 	gen.dumptimes(img,scale=1.1)
# 	
# 	sumlist=calibcam(n,cam)
# 	std=stats.tstd(sumlist)
# 	avg=sum(sumlist)/n
# 	framenum=0
# 	oldids=c.ids
# 	
# 	print "starting detection"
# 	
# 	while True:
# 		retval,img=cam.read()
# 		xstart=img.shape[0]/4
# 		ystart=img.shape[1]/4
# 		thisz=(img[xstart:(xstart*3),ystart:(ystart*3),:].sum()-avg)/std
# 		#while True:
# 		#	print "testing for sleep needed"
# 		#	try:
# 		#		if subsocket.recv(flags=zmq.NOBLOCK)=="need quite? yes":
# 		#			print "need sleep"
# 		#			while True:
# 		#				try:
# 		#					if subsocket.recv(flags=zmq.NOBLOCK)=="need quite? not anymore":
# 		#						break
# 		#				except:
# 		#					pass
# 		#				time.sleep(.5)
# 		#	except:
# 		#		break
# 		if abs(thisz)>threshold:
# 			print "something weird, zscore="+str(thisz)
# 			time.sleep(.5)
# 		#	pubsocket.send("need quiet? yes")
# 			retval,newimg=cam.read() #works well on mac, maybe not pis...
# 			#if not len(c.ids)==len(oldids): #done periodically, not ideal to update here...
# 			#	print "cluster changed"
# 			#	numnodes=len(c.ids)
# 			#	dview=c[:]
# 			#	print "repredicting even distribution..."
# 			#	predictdist(numnodes,times,numsizes,c)
# 			#	for id in c.ids:
# 			#		if id not in oldids:
# 			#			c[id].execute('import cv2,sys,time,math,os,cPickle,signal;myfaces=[]')
# 			#			c[id]['getrunstate']=getrunstate
# 			#			c[id]['getmypid']=getmypid
# 			#			c[id]['scale']=scale
# 			#			c[id].execute('haarface=cv2.CascadeClassifier("/home/pi/opencv-2.4.9/data/haarcascades/haarcascade_frontalface_alt2.xml")')
# 					#else:
# 					#	c[id].execute('myfaces=[]')
# 			#	oldids=c.ids
# 			print "searching for faces..."
# 			faces=findfaces(newimg,scale,serial)
# 		#	pubsocket.send("need quiet? not anymore")
# 			if len(faces)>0:
# 				print "FOUND A FACEZ!!!!!11!"
# 				if show==True:
# 					for (x,y,h,w) in faces:
# 						cv2.rectangle(newimg,(x,y),(x+w,y+h),(0,255,255),1)
# 					#cv2.imshow("obj num "+str(i),newimg)
# 					cv2.imshow("obj found",newimg)
# 			else:
# 				print "no facez :("
# 				if show==True:
# 					cv2.imshow("obj not found",newimg)
# 			if show==True:
# 				cv2.waitKey(1)
# 			sumlist[0]=img[xstart:(xstart*3),ystart:(ystart*3),:].sum()
# 			std=stats.tstd(sumlist)
# 			avg=sum(sumlist)/n
# 		#else:
# 		#	pubsocket.send("need quiet? no")
# 		framenum=framenum+1
# 		if framenum%10==0:
# 			sumlist[0]=img[xstart:(xstart*3),ystart:(ystart*3),:].sum()
# 			std=stats.tstd(sumlist)
# 			avg=sum(sumlist)/n
# 		#if framenum%100==0:
# 		#	print framenum
# 		if framenum%(10*numnodes)==0:
# 			if not len(c.ids)==len(oldids):
# 				print "cluster changed"
# 				numnodes=len(c.ids)
# 				dview=c[:]
# 				print "repredicting even distribution..."
# 				predictdist(numnodes,times,numsizes,c)
# 				for id in c.ids:
# 					if id not in oldids:
# 						c[id].execute('import cv2,sys,time,math,os,cPickle,signal;myfaces=[]')
# 						c[id]['getrunstate']=getrunstate
# 						c[id]['getmypid']=getmypid
# 						c[id]['scale']=scale
# 						c[id].execute('haarface=cv2.CascadeClassifier("/home/pi/opencv-2.4.9/data/haarcascades/haarcascade_frontalface_alt2.xml")')
# 					#else:
# 					#	c[id].execute('myfaces=[]')
# 				oldids=c.ids
# 				#c.spin()
# 				#trying reseting the client
# 				c=Client(profile=profile)
# 				dview=c[:]
# 
# 
# def getalignface(facefinder,eyefinder,nosefinder,cam=False,img=[]):
# 	cv2.destroyAllWindows()
# 	if cam:
# 		print "finding faces..."
# 		retval,img=cam.read()
# 		if not retval:
# 			print "could not read image"
# 	elif len(img)==0:
# 		print "if not cam, must have img"
# 	cv2.imshow("img",img)
# 	cv2.waitKey(25)
# 	#cv2.imshow("img",img)
# 	dims=facefinder.detectMultiScale(img,1.1,5,1,(20,20),img.shape[:2])
# 	print dims
# 	if len(dims)>0:
# 		sizelist=[w for (x,y,h,w) in dims]
# 		maxsize=max(sizelist)
# 		biggestface=[[x,y,h,w] for (x,y,h,w) in dims if w==maxsize and w>60]
# 		print biggestface
# 		if len(biggestface)>0:
# 			print "finding facial features..."
# 			for (x,y,h,w) in biggestface:
# 				thisface=img[x:(x+w),y:(y+h),:]
# 				cleanface=copy.deepcopy(thisface)
# 			cv2.imshow("thisface",thisface)
# 			eyes=eyefinder.detectMultiScale(thisface,1.1,30,1)
# 			noses=nosefinder.detectMultiScale(thisface,1.1,20,1)
# 			print "eyes: "+str(eyes)
# 			print "noses: "+str(noses)
# 			if len(noses)>0 and len(eyes)>1:
# 				maxnosesize=max([w for (x,y,h,w) in noses])
# 				biggestnose=[[x,y,h,w] for (x,y,h,w) in noses if w==maxnosesize][0]
# 				finaleyes=[]
# 				print "filtering out nostrils from the eyes..."
# 				for eye in eyes:
# 					if not (eye[0]>(biggestnose[0]-5) and eye[0]<(biggestnose[0]+biggestnose[3]+5) and eye[1]>(biggestnose[1]-5) and eye[1]<(biggestnose[1]+biggestnose[2]+5)):
# 						finaleyes.append(eye)
# 				print "eyes: "+str(finaleyes)
# 				print "noses: "+str(noses)
# 				for (x,y,h,w) in finaleyes:
# 					cv2.rectangle(thisface,(x,y),(x+w,y+h),(0,255,255),1)
# 				for (x,y,h,w) in [biggestnose]:
# 					cv2.rectangle(thisface,(x,y),(x+w,y+h),(255,255,0),1)
# 				#print biggestnose
# 			elif len(eyes)>1 and len(eyes)<3:
# 				print "could not find a nose, found 2 eyes. working with them"
# 				for (x,y,h,w) in eyes:
# 					cv2.rectangle(thisface,(x,y),(x+w,y+h),(0,255,255),1)
# 				cv2.imshow("thisface",thisface)
# 				finaleyes=eyes
# 			else: #add parsing for more than 2 detected eyes without a nose
# 				print "feature detection went wrong"
# 				return []
# 			cv2.imshow("thisface features",thisface)
# 			eye1mid=(finaleyes[0][0]+finaleyes[0][3]/2,finaleyes[0][1]+finaleyes[0][2]/2)
# 			try:
# 				eye2mid=(finaleyes[1][0]+finaleyes[1][3]/2,finaleyes[1][1]+finaleyes[1][2]/2)
# 			except:
# 				print "something went wrong with the format of eyes, skipping to next iteration"
# 				return []
# 			if eye1mid[0]<eye2mid[0]:
# 				lefteye=eye1mid
# 				righteye=eye2mid
# 			else:
# 				lefteye=eye2mid
# 				righteye=eye1mid
# 			borderoffset=9.0/12.0
# 			midborderoffset=3
# 			eyetriangle=[righteye[0]-lefteye[0],righteye[1]-lefteye[1]]
# 			topbordermid=[lefteye[0]+eyetriangle[0]/2+eyetriangle[1]/midborderoffset,lefteye[1]+eyetriangle[1]/2-eyetriangle[0]/midborderoffset]
# 			topborder=[(topbordermid[0]-int(eyetriangle[0]*borderoffset),topbordermid[1]-int(eyetriangle[1]*borderoffset)),(topbordermid[0]+int(eyetriangle[0]*borderoffset),topbordermid[1]+int(eyetriangle[1]*borderoffset))]
# 			bordertriangle=[topborder[1][0]-topborder[0][0],topborder[1][1]-topborder[0][1]]
# 			borderlength=int(np.sqrt(bordertriangle[0]**2+bordertriangle[1]**2))
# 			print bordertriangle
# 			border=[topborder[0],topborder[1],(topborder[1][0]-bordertriangle[1],topborder[1][1]+bordertriangle[0]),(topborder[0][0]-bordertriangle[1],topborder[0][1]+bordertriangle[0])]
# 			violation=0
# 			for i in range(4):
# 				for j in range(2):
# 					if border[i][j]>thisface.shape[0] or border[i][j]<0:
# 						violation=1			
# 			if violation:
# 				print "could not align face, features too close to edge"
# 			else:
# 				print "aligning face"
# 				cv2.line(thisface,eye1mid,eye2mid,(255,0,0),1)
# 				cv2.line(thisface,border[0],border[1],(0,255,0),1)
# 				cv2.line(thisface,border[2],border[3],(0,255,0),1)
# 				cv2.line(thisface,border[0],border[3],(0,255,0),1)
# 				cv2.line(thisface,border[1],border[2],(0,255,0),1)
# 				#cv2.line(thisface,(nosemid[0]+eyedist,nosemid[1]+noseedgeoffset),(nosemid[0]-eyedist,nosemid[1]+noseedgeoffset),(0,255,0),1)
# 				facecenter=tuple(np.array([border[0][0]+border[2][0],border[0][1]+border[2][1]])/2)
# 				cv2.circle(thisface,facecenter,5,(255,0,0),-1)
# 				cv2.imshow("thisface border",thisface)
# 				rotationangle=np.arctan(float(eyetriangle[1])/float(eyetriangle[0]))
# 				rotationmat=cv2.getRotationMatrix2D(facecenter,rotationangle*180/np.pi,1.0)
# 				print border[0]
# 				bordercenteroffset=[border[0][0]-facecenter[0],border[0][1]-facecenter[1]]
# 				print bordercenteroffset
# 				borderoffsetafterrotation=[bordercenteroffset[0]*np.cos(rotationangle)+bordercenteroffset[1]*np.sin(rotationangle),bordercenteroffset[0]*np.sin(rotationangle)-bordercenteroffset[1]*np.cos(rotationangle)-borderlength]
# 				print borderoffsetafterrotation
# 				borderafterrotation=(int(borderoffsetafterrotation[0]+facecenter[0]),int(borderoffsetafterrotation[1]+facecenter[1]))
# 				print borderafterrotation
# 				finalface=cv2.warpAffine(thisface,rotationmat,thisface.shape[:2],flags=cv2.INTER_LINEAR)
# 				finalcroppedface=finalface[borderafterrotation[1]:borderlength+borderafterrotation[1],borderafterrotation[0]:borderafterrotation[0]+borderlength]
# 				cv2.circle(finalface,borderafterrotation,3,(0,255,0),-1)
# 				#cv2.circle(finalface,(borderafterrotation[0],borderafterrotation[1]-borderlength),3,(0,255,0),-1)
# 				cv2.circle(finalface,facecenter,4,(0,255,255),-1)
# 				cv2.imshow("rotated",finalface)
# 				cv2.imshow("final cropped face",finalcroppedface)
# 				cleanface=cv2.warpAffine(cleanface,rotationmat,thisface.shape[:2],flags=cv2.INTER_LINEAR)[borderafterrotation[1]:borderlength+borderafterrotation[1],borderafterrotation[0]:borderafterrotation[0]+borderlength]
# 				cv2.imshow("clean face",cleanface)
# 				cleanface=cv2.cvtColor(cleanface,cv2.COLOR_BGR2GRAY)
# 				cleanface=cv2.resize(cleanface,(60,60))
# 				cv2.imshow("small clean face",cleanface)
# 				return cleanface
# 			while 1:
# 				k=cv2.waitKey(10)
# 				if k==27:
# 					break
# 				else:
# 					continue
# 	return []
# 
# 
# def makepredictor():
# 	imgset=[]
# 	labels=[]
# 	facefinder=cv2.CascadeClassifier("/home/pi/opencv-2.4.9/data/haarcascades/haarcascade_frontalface_alt2.xml")
# 	eyefinder=cv2.CascadeClassifier("/home/pi/opencv-2.4.9/data/haarcascades/haarcascade_eye.xml")
# 	nosefinder=cv2.CascadeClassifier("/home/pi/opencv-2.4.9/data/haarcascades/haarcascade_mcs_nose.xml")
# 	cam=cv2.VideoCapture(-1)
# 	while not raw_input("capture image? y/n ")=='n':
# 		face=getalignface(facefinder,eyefinder,nosefinder,cam)
# 		if len(face)>0:
# 			if not raw_input("is this the correct face? y/n ")=='n':
# 				imgset.append(face)
# 				labels.append(int(raw_input("who is this? (int) ")))
# 
# 	labels=np.array(labels)
# 	facerec=cv2.createLBPHFaceRecognizer()
# 	facerec.train(imgset,labels)
# 	return facerec


import cv2, time, sys, cPickle
from math import log
def gentimedist(imgname,slices=False,scale=1.1,invert=0):
	if sys.platform=='darwin':
		if type(imgname)=='str':
			img=cv2.imread("/Users/isaac/Code/"+imgname)#raw_input("image? "))
		else:
			img=imgname
		haarface=cv2.CascadeClassifier("/usr/local/Cellar/opencv/2.4.9/share/OpenCV/haarcascades/haarcascade_frontalface_alt2.xml")
	elif sys.platform=='linux2':
		if type(imgname)=='str':
			img=cv2.imread("/home/pi/"+imgname)
		else:
			img=imgname
		haarface=cv2.CascadeClassifier("/home/pi/opencv-2.4.9/data/haarcascades/haarcascade_frontalface_alt2.xml")
	#numstages=10
	print "got here (yet again)"
	try:
		sp=img.shape
		sp=min(sp[:1])
	except:
		print "could not read image"
		sys.exit("could not read image")
	if not slices:
		slices=int(log(sp/20)/log(scale))
		uselog=True
	else:
		uselog=False
	if slices>sp:
		print "too many slices, using sp"
		slices=sp
	inc=(sp-20)/slices
	trans=[None]*slices
	times=[None]*(slices-1)
	if uselog:
		for i in range(slices):
			trans[i]=int(20*(scale**i))
		#print trans
	else:
		for i in range(slices):
			trans[i]=int(inc*(i))+20
	for i in range(slices-1):
		times[i]=time.time()
		haarface.detectMultiScale(img,scale,1,1,tuple([trans[i]-1]*2),tuple([trans[i]+1]*2))
		times[i]=(time.time()-times[i])
		if invert:
			times[i]=1/times[i]**2
	print max(times)
	print min(times)
	print sum(times)/len(times)
	return times
def dumptimes(imgname,filename="timedistinfo",slices=False,scale=1.01,invert=0):
	times=gentimedist(imgname,slices,scale,invert)
	file=open(filename,"w")
	cPickle.dump(times,file)
	file.close()
	print "dumped time info to "+filename
def loadtimes(filename="timedistinfo"):
	file=open(filename,"r")
	times=cPickle.load(file)
	file.close()
	return times
def findratios(imgname,ratios=1,mag=10,slices=100,scale=1.1,invert=0):
	metatimes=[[None]]*(ratios+1)
	for i in range(ratios+1):
		metatimes[i]=gentimedist(imgname,slices,((scale-1)*(mag**(-i))+1),invert)
	for i in range(ratios):
		thisratio=[None]*(slices-1)
		for j in range(slices-1):
			thisratio[j]=(metatimes[i+1][j]-min(metatimes[i+1])+.05)/(metatimes[i][j]-min(metatimes[i])+.05)
		print thisratio
		print max(thisratio)
		print min(thisratio)
		print sum(thisratio)/(slices-1)


def genmyspotlist0(myspot): #-1-1 biased. need to bias according to start
	myspot=list(myspot)
	myspotlist=[None]*8
	myspotlist[0]=(myspot[0]-1,myspot[1]-1)
	myspotlist[1]=(myspot[0],myspot[1]-1)
	myspotlist[3]=(myspot[0]+1,myspot[1]-1)
	myspotlist[2]=(myspot[0]-1,myspot[1])
	myspotlist[5]=(myspot[0]+1,myspot[1])
	myspotlist[4]=(myspot[0]-1,myspot[1]+1)
	myspotlist[6]=(myspot[0],myspot[1]+1)
	myspotlist[7]=(myspot[0]+1,myspot[1]+1)
	return myspotlist
def genmyspotlist7(myspot): #+1+1 biased. need to bias according to start
	myspot=list(myspot)
	myspotlist=[None]*8
	myspotlist[7]=(myspot[0]-1,myspot[1]-1)
	myspotlist[6]=(myspot[0],myspot[1]-1)
	myspotlist[4]=(myspot[0]+1,myspot[1]-1)
	myspotlist[5]=(myspot[0]-1,myspot[1])
	myspotlist[2]=(myspot[0]+1,myspot[1])
	myspotlist[3]=(myspot[0]-1,myspot[1]+1)
	myspotlist[1]=(myspot[0],myspot[1]+1)
	myspotlist[0]=(myspot[0]+1,myspot[1]+1)
	return myspotlist
def genmyspotlist5(myspot): #-1+1 biased. need to bias according to start
	myspot=list(myspot)
	myspotlist=[None]*8
	myspotlist[3]=(myspot[0]-1,myspot[1]-1)
	myspotlist[5]=(myspot[0],myspot[1]-1)
	myspotlist[7]=(myspot[0]+1,myspot[1]-1)
	myspotlist[2]=(myspot[0]-1,myspot[1])
	myspotlist[6]=(myspot[0]+1,myspot[1])
	myspotlist[0]=(myspot[0]-1,myspot[1]+1)
	myspotlist[1]=(myspot[0],myspot[1]+1)
	myspotlist[4]=(myspot[0]+1,myspot[1]+1)
	return myspotlist
def genmyspotlist2(myspot): #+1-1 biased. need to bias according to start
	myspot=list(myspot)
	myspotlist=[None]*8
	myspotlist[4]=(myspot[0]-1,myspot[1]-1)
	myspotlist[1]=(myspot[0],myspot[1]-1)
	myspotlist[0]=(myspot[0]+1,myspot[1]-1)
	myspotlist[6]=(myspot[0]-1,myspot[1])
	myspotlist[2]=(myspot[0]+1,myspot[1])
	myspotlist[7]=(myspot[0]-1,myspot[1]+1)
	myspotlist[5]=(myspot[0],myspot[1]+1)
	myspotlist[3]=(myspot[0]+1,myspot[1]+1)
	return myspotlist
def genmyspotlist1(myspot): #0-1 biased. need to bias according to start
	myspot=list(myspot)
	myspotlist=[None]*8
	myspotlist[2]=(myspot[0]-1,myspot[1]-1)
	myspotlist[0]=(myspot[0],myspot[1]-1)
	myspotlist[1]=(myspot[0]+1,myspot[1]-1)
	myspotlist[3]=(myspot[0]-1,myspot[1])
	myspotlist[4]=(myspot[0]+1,myspot[1])
	myspotlist[6]=(myspot[0]-1,myspot[1]+1)
	myspotlist[7]=(myspot[0],myspot[1]+1)
	myspotlist[5]=(myspot[0]+1,myspot[1]+1)
	return myspotlist
def genmyspotlist6(myspot): #0+1 biased. need to bias according to start
	myspot=list(myspot)
	myspotlist=[None]*8
	myspotlist[5]=(myspot[0]-1,myspot[1]-1)
	myspotlist[7]=(myspot[0],myspot[1]-1)
	myspotlist[6]=(myspot[0]+1,myspot[1]-1)
	myspotlist[4]=(myspot[0]-1,myspot[1])
	myspotlist[3]=(myspot[0]+1,myspot[1])
	myspotlist[1]=(myspot[0]-1,myspot[1]+1)
	myspotlist[0]=(myspot[0],myspot[1]+1)
	myspotlist[2]=(myspot[0]+1,myspot[1]+1)
	return myspotlist
def genmyspotlist4(myspot): #+10 biased. need to bias according to start
	myspot=list(myspot)
	myspotlist=[None]*8
	myspotlist[6]=(myspot[0]-1,myspot[1]-1)
	myspotlist[3]=(myspot[0],myspot[1]-1)
	myspotlist[2]=(myspot[0]+1,myspot[1]-1)
	myspotlist[7]=(myspot[0]-1,myspot[1])
	myspotlist[0]=(myspot[0]+1,myspot[1])
	myspotlist[5]=(myspot[0]-1,myspot[1]+1)
	myspotlist[4]=(myspot[0],myspot[1]+1)
	myspotlist[1]=(myspot[0]+1,myspot[1]+1)
	return myspotlist
def genmyspotlist3(myspot): #-10 biased. need to bias according to start
	myspot=list(myspot)
	myspotlist=[None]*8
	myspotlist[1]=(myspot[0]-1,myspot[1]-1)
	myspotlist[4]=(myspot[0],myspot[1]-1)
	myspotlist[5]=(myspot[0]+1,myspot[1]-1)
	myspotlist[0]=(myspot[0]-1,myspot[1])
	myspotlist[7]=(myspot[0]+1,myspot[1])
	myspotlist[2]=(myspot[0]-1,myspot[1]+1)
	myspotlist[3]=(myspot[0],myspot[1]+1)
	myspotlist[6]=(myspot[0]+1,myspot[1]+1)
	return myspotlist
#dview['genmyspotlist']=genmyspotlist
def createcenter(layer,midspot):
	midspot=list(midspot)
	layerspots=[None]*(8*(layer-1))
	def varyx(mult=1):
		thissection=[]
		for x in range(-1*(layer-1),layer+1):
			thissection.append(tuple([midspot[0]+mult*x,midspot[1]+mult*layer]))
		return thissection
	def varyy(mult=1):
		thissection=[]
		for y in range(-1*(layer-1),layer+1):
			thissection.append(tuple([midspot[0]+mult*layer,midspot[1]+-1*mult*y]))
		return thissection
	layerspots[0:2*(layer-1)-1]=varyx()
	layerspots[2*(layer-1):4*(layer-1)-1]=varyy()
	layerspots[4*(layer-1):6*(layer-1)-1]=varyx(mult=-1)
	layerspots[6*(layer-1):8*(layer-1)-1]=varyy(mult=-1)
	return layerspots
def nearestzero(myspot,map):
	myspotlist=genmyspotlist(myspot)
	listofbadspots=[]
	for i in range(0,8):
		if len(map) in myspotlist[i] or -1 in myspotlist[i]:
			listofbadspots.append(myspotlist[i])
	for i in listofbadspots:
		myspotlist.remove(i)
	x,y=numpy.where(map==0)
	zerospots=zip(x,y)
	if len(zerospots)==0:
		print 'could not find any zeros, will stay put'
		return myspot
	zerodists=[None]*len(zerospots)
	for i in range(0,len(zerospots)):
		zerodists[i]=sqrt((myspot[0]-zerospots[i][0])**2+(myspot[1]-\
		       zerospots[i][1])**2)
	closestzero=zerospots[zerodists.index(min(zerodists))]
	distlist=[None]*len(myspotlist)
	for i in range(0,len(myspotlist)):
		distlist[i]=sqrt((closestzero[0]-myspotlist[i][0])**2+\
		         (closestzero[1]-myspotlist[i][1])**2)
	return tuple(myspotlist[distlist.index(min(distlist))])

def updatemap():
	for spot in spots:
		background[spot[0],spot[1]]=background[spot[0],spot[1]]+1;

#define different search methods and push to engines
def purgelist(myspotlist,map,nope=False,best=False):
	if nope:
		if best:
			myspotlist.remove(nope)
		else:
			for i in range(len(nope)):
				if not len(nope[i])==2:
					aowrungoaew
				if nope[i] in myspotlist:
					myspotlist.remove(nope[i])
	listofbadspots=[]
	for i in range(0,len(myspotlist)):
		if len(map) in myspotlist[i] or -1 in myspotlist[i] or map[\
		             myspotlist[i][0],myspotlist[i][1]]!=0:
			listofbadspots.append(myspotlist[i])
	for i in listofbadspots:
		myspotlist.remove(i)
	return myspotlist

def edgeseek(center,myspot,map,nope=False):
	if type(myspot[0])==tuple:
		myspot=myspot[0]
	oktogo=0
	repeats=0
	myspotlist=genmyspotlist(myspot)
	#if nope:
	#	if len(nope)>2:
	#		if nope[len(nope)-2]==nope[len(nope)-1]:
	#			nope=nope[len(nope)-1]
	myspotlist=purgelist(myspotlist,map,nope)
	if len(myspotlist)==0:
		return [nearestzero(myspot,map),0]
	#if len(myspotlist)==1:
	#	return list(tuple(myspotlist[0]),1)
	distlist=[None]*len(myspotlist)
	for i in range(len(distlist)):
		distlist[i]=sqrt((center[0]-myspotlist[i][0])**2+(center[1]-\
		            myspotlist[i][1])**2)
	bestspot=tuple(myspotlist[distlist.index(max(distlist))])
	bestspotlist=purgelist(genmyspotlist(bestspot),map,myspot,1)
	locklist=[]
	needtoloop=len(bestspotlist)==1# and bestspotlist[0] in myspotlist
	oldbestspots=[]
	locklist=[]
	while needtoloop:
		if not len(locklist)==0:
			oldbestspots.append(deepbestspot)
		else:
			oldbestspots.append(myspot)
			oldbestspots.append(bestspot)
		deepbestspot=bestspotlist[0]
		locklist.append(deepbestspot)
		bestspotlist=purgelist(genmyspotlist(deepbestspot),map,oldbestspots)
		for spot in locklist:
			if spot in bestspotlist:
				bestspotlist.remove(spot)
		if len(bestspotlist)==1:
			deepbestspot=bestspotlist[0]
		else:
			needtoloop=0
		#return [bestspot,tuple(bestspotlist[0])]
	#if nope:
	#	if len(nope)==1:
	#		sys.stdout.write("something has gone terribly wrong"+str(nope))
	#		if list(bestspot)==list(nope[0]):
	#			aosinruaonergapojrg
	if len(locklist)>0:
		return [bestspot,tuple(locklist)]
	else:
		return [bestspot,0]
	#if len(myspotlist)==1:
	#	return [bestspot,myspotlist[0]]
	#return [bestspot,0]

def centerseek(center,myspot,map,nope=False):
	if type(myspot[0])==tuple:
		myspot=myspot[0]
	oktogo=0
	repeats=0
	#for now, redefining center to be center of map
	
	myspotlist=genmyspotlist(myspot)
	if nope:
		myspotlist.remove(nope)
	listofbadspots=[]
	for i in range(0,len(myspotlist)):
		if len(map) in myspotlist[i] or -1 in myspotlist[i] or map[\
		             myspotlist[i][0],myspotlist[i][1]]!=0:
			listofbadspots.append(myspotlist[i])
	for i in listofbadspots:
		myspotlist.remove(i)
	if len(myspotlist)==0:
		return nearestzero(myspot,map)
	#if len(myspotlist)==1:
	#	return list(tuple(myspotlist[0]),1)
	if len(myspotlist)==1:
		print "only one option"
		return tuple(myspotlist[0])
	distlist=[None]*len(myspotlist)
	for i in range(0,len(distlist)):
		distlist[i]=sqrt((center[0]-myspotlist[i][0])**2+(center[1]-\
		            myspotlist[i][1])**2)
	return tuple(myspotlist[distlist.index(min(distlist))])

